#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <ctype.h>   // rdg for isalnum()
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "misc.h"


gboolean
quit_callback                          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_exit(1);
  return FALSE;
}


void
on_new_file_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *listbox;
  
  currentFile = NULL;
  clear_interface(NULL);
  listbox = lookup_widget(GTK_WIDGET(refholder),"list_snort_rules");
  gtk_list_clear_items(GTK_LIST(listbox),0,-1);
}

void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *file_selection;
  
  operation = "open";
  file_selection = create_file_selection();
  gtk_file_selection_complete(GTK_FILE_SELECTION(file_selection),"*.rules");
  gtk_widget_show(file_selection);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *file_selection;

  if (currentFile != NULL) {
    operation = "save";
    file_prep((const char *)currentFile);
  }
  else {
    operation = "save_as";
    file_selection = create_file_selection();
    gtk_file_selection_complete(GTK_FILE_SELECTION(file_selection),"*.rules");
    gtk_widget_show(file_selection);
  }
}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *file_selection;
  
  file_selection = create_file_selection();
  gtk_file_selection_complete(GTK_FILE_SELECTION(file_selection),"*.rules");
  gtk_widget_show(file_selection);
  operation = "save_as";
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  //  GtkWidget *editable;
  

}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  clear_interface(NULL);
}


void
on_options_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  /* not yet implemented.  save for editing snort configuration file in next ver. */
  
  GtkWidget *snort_options = create_snort_options();
  gtk_widget_show(snort_options);
}


void
on_manual_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
					  ("Sorry, this feature not implemented yet.")));
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about_box = create_about_box();  
  gtk_widget_show(about_box);
}


void
on_ok_file_select_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *listbox;
 
  currentFile = gtk_file_selection_get_filename(GTK_FILE_SELECTION(user_data));
  
  if (!g_strcasecmp(operation, "open")) {
    clear_interface(NULL);
    listbox = lookup_widget(GTK_WIDGET(refholder),"list_snort_rules");
    gtk_list_clear_items(GTK_LIST(listbox),0,-1);
    file_opener((const char *)currentFile); 
  }  
  else if ((g_strcasecmp(operation,"save") == 0) || (g_strcasecmp(operation,"save_as") == 0)) {
    file_prep((const char *)currentFile);
  }
  gtk_widget_hide(gtk_widget_get_toplevel(GTK_WIDGET(button)));  
}
 

void
on_cancel_file_select_button_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

void
on_button_open_ruleset_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *file_selection;

  operation = "open";
  file_selection = create_file_selection();
  gtk_file_selection_complete(GTK_FILE_SELECTION(file_selection),"*.rules");
  gtk_widget_show(file_selection);
}


void
on_sneakymain_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
  currentFile = NULL;
  refholder = widget;
}


void
on_list_snort_rules_select_child       (GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
  gchar *ruleString, *optionKey, *optionVal;
  GtkWidget *helper;  // temp. widget for lookup_widget() so i don't have to make tons.
  
  operation = "rule_select";
  currentListItem = widget;
  gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child),&ruleString);
  rule_parser(ruleString);

  helper = lookup_widget(GTK_WIDGET(refholder), "menu_action");
  if (g_strcasecmp(current.action,"alert") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),0);
  else if (g_strcasecmp(current.action,"log") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),1);
  else if (g_strcasecmp(current.action,"pass") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),2);
  else if (g_strcasecmp(current.action,"activate") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),3);
  else if (g_strcasecmp(current.action,"dynamic") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),4);
  else {
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
					    ("Error in action field - typo?")));
    return;			    
  }
  
  helper = lookup_widget(GTK_WIDGET(refholder), "menu_protocol");
  if (g_strcasecmp(current.protocol,"ip") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),0);
  else if (g_strcasecmp(current.protocol,"icmp") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),1);
  else if (g_strcasecmp(current.protocol,"tcp") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),2);
  else if (g_strcasecmp(current.protocol,"udp") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),3);
  else {
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
					    ("Error in protocol field - typo?")));
    return ;
  }
  
  helper = lookup_widget(GTK_WIDGET(refholder),"entry_source");
  gtk_entry_set_text(GTK_ENTRY(helper),current.src_ip);
  
  helper = lookup_widget(GTK_WIDGET(refholder),"entry_src_ports");
  gtk_entry_set_text(GTK_ENTRY(helper),current.src_port);
  
  helper = lookup_widget(GTK_WIDGET(refholder),"menu_direction");
  if (g_strcasecmp(current.direction,"->") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),0);
  else if (g_strcasecmp(current.direction,"<>") == 0)
    gtk_option_menu_set_history(GTK_OPTION_MENU(helper),1);
  else {
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
					    ("Error in direction field - typo?")));
    return;
  }
  
  helper = lookup_widget(GTK_WIDGET(refholder),"entry_dest");
  gtk_entry_set_text(GTK_ENTRY(helper),current.dest_ip);

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_dest_ports");
  gtk_entry_set_text(GTK_ENTRY(helper),current.dest_port);
 
  helper = lookup_widget(GTK_WIDGET(refholder),"combo_options");
  optionKey = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(helper)->entry));
  optionVal = g_hash_table_lookup(current.rule_options,optionKey);
  
  if (optionVal != NULL) {
    optionVal = g_hash_table_lookup(current.rule_options,optionKey);
    helper = lookup_widget(GTK_WIDGET(refholder),"entry_options_value");
    gtk_entry_set_text(GTK_ENTRY(helper),optionVal);
  }
  else {
    helper = lookup_widget(GTK_WIDGET(refholder),"entry_options_value");
    gtk_editable_delete_text(GTK_EDITABLE(helper),0,-1);
  }  
}


void
on_button_option_enter_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *combo_box, *value_entry;
  gchar *optionKey = NULL, *optionVal = NULL;
  gpointer *KeyRef = NULL, *ValRef = NULL;
  struct options_struct *new_option = NULL, *option_part = NULL;
  GSList *option_list = NULL;
  gboolean found = FALSE;

  if (current.rule_options == NULL)
    current.rule_options = g_hash_table_new(g_str_hash, g_str_equal);

  combo_box = lookup_widget(GTK_WIDGET(refholder),"combo_options");
  optionKey = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo_box)->entry));
  
  if (optionKey != NULL) {
    // rdg check that this is a non whitespace entry if not throw up an error box
    if (!isalnum(optionKey[0])) {
      /* FIXME throw up the error box */
      return;
    }
    value_entry = lookup_widget(GTK_WIDGET(refholder),"entry_options_value");
    optionVal = gtk_entry_get_text(GTK_ENTRY(value_entry));
    if (!optionVal || !isalnum(optionVal[0]))
      optionVal = NULL;
    if (!optionVal) {
      printf("in the null\n");
      if (g_hash_table_lookup_extended(current.rule_options, optionKey, KeyRef, ValRef)) {
        /* check to see if the optionKey is in the option_matrix */
        /* we know key is in the hash table, have to free it from the options_matrix */
        for (option_list = current.options_matrix; option_list; option_list = option_list->next) {
          option_part = option_list->data;
          if (!strcmp(option_part->keyword, optionKey)) {
            found = TRUE;
            break;
  }
        }
        if (found) { /* we know option_list points at what we want to remove */
          current.options_matrix = g_slist_remove_link(current.options_matrix, option_list);
          g_hash_table_remove(current.rule_options, option_part->keyword);
          g_slist_free_1(option_list);
        }
      }
    } else {
      if (g_hash_table_lookup_extended(current.rule_options, optionKey, KeyRef, ValRef)) {
        /* we know key is in the hash table, have to free it from the options_matrix */
        for (option_list = current.options_matrix; option_list; option_list = option_list->next) {
          option_part = option_list->data;
          if (!strcmp(option_part->keyword, optionKey)) {
            found = TRUE;
            break;
          }
        }
        if (found) { /* we know option_list points at what we want to remove */
          current.options_matrix = g_slist_remove_link(current.options_matrix, option_list);
          g_hash_table_remove(current.rule_options, optionKey);
          g_slist_free_1(option_list);
        }
      }
      new_option = g_new(struct options_struct, 1);
      new_option->keyword = g_strdup(optionKey);
      new_option->value = g_strdup(optionVal);
      current.options_matrix = g_slist_append(current.options_matrix, new_option);
      g_hash_table_insert(current.rule_options, new_option->keyword, new_option->value);
    }
  }
//  optionVal = g_hash_table_lookup(current.rule_options,optionKey);
}


void
on_button_remove_rule_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *list_rules;

  list_rules = lookup_widget(GTK_WIDGET(refholder),"list_snort_rules");
  gtk_container_remove(GTK_CONTAINER(list_rules),currentListItem);
}


void
on_button_add_rule_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  gchar *ruleString;
  GtkWidget *helper, *item;
  GtkWidget *option_menu, *menu, *active_item;
  gint active_index;

  option_menu = lookup_widget(GTK_WIDGET(refholder),"menu_action");
  menu = GTK_OPTION_MENU(option_menu)->menu;
  active_item = gtk_menu_get_active(GTK_MENU(menu));
  active_index = g_list_index(GTK_MENU_SHELL(menu)->children,active_item);

  switch (active_index) {
    case 0:current.action = "alert";
      break;
    case 1:current.action = "log";
      break;
    case 2:current.action = "pass";
      break;
    case 3:current.action = "activate";
      break;
    case 4:current.action = "dynamic";
      break;
    default: // should never get here
    }
 
  option_menu = lookup_widget(GTK_WIDGET(refholder),"menu_protocol");
  menu = GTK_OPTION_MENU(option_menu)->menu;
  active_item = gtk_menu_get_active(GTK_MENU(menu));
  active_index = g_list_index(GTK_MENU_SHELL(menu)->children,active_item);
  
  switch (active_index) {
  case 0:current.protocol = "ip";
    break;
  case 1:current.protocol = "icmp";
    break;
  case 2:current.protocol = "tcp";
    break;
  case 3:current.protocol = "udp";
    break;
  default: // should never get here
    break;
  }

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_source");
  current.src_ip = gtk_entry_get_text(GTK_ENTRY(helper));

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_src_ports");
  current.src_port = gtk_entry_get_text(GTK_ENTRY(helper));

  option_menu = lookup_widget(GTK_WIDGET(refholder),"menu_direction");
  menu = GTK_OPTION_MENU(option_menu)->menu;
  active_item = gtk_menu_get_active(GTK_MENU(menu));
  active_index = g_list_index(GTK_MENU_SHELL(menu)->children,active_item);

  switch (active_index) {
  case 0:current.direction = "->";
    break;
  case 1:current.direction = "<>";
    break;
  default:
    break; 
  }

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_dest");
  current.dest_ip = gtk_entry_get_text(GTK_ENTRY(helper));

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_dest_ports");
  current.dest_port = gtk_entry_get_text(GTK_ENTRY(helper));

  ruleString = rule_builder(NULL);
  if (currentListItem == NULL) {
    helper = lookup_widget(GTK_WIDGET(refholder), "list_snort_rules");
    if (helper != NULL) {
      item = gtk_list_item_new_with_label(ruleString);
      gtk_container_add(GTK_CONTAINER(helper), item);
      gtk_widget_show(item);
      currentListItem = item;
    }
  }
  else
    gtk_label_set_text(GTK_LABEL(GTK_BIN(currentListItem)->child),ruleString);
}  



void
on_entry_combo_keyword_activate        (GtkEditable     *editable,
                                        gpointer         user_data)
{
 GtkWidget *combo_box, *value_entry;
  gchar *optionKey, *optionVal;

  combo_box = lookup_widget(GTK_WIDGET(refholder),"combo_options");
  optionKey = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo_box)->entry));
  optionVal = g_hash_table_lookup(current.rule_options,optionKey);
  
  if (optionVal != NULL) {
    value_entry = lookup_widget(GTK_WIDGET(refholder),"entry_options_value");
    gtk_entry_set_text(GTK_ENTRY(value_entry),optionVal);
  }
  else {
    value_entry = lookup_widget(GTK_WIDGET(refholder),"entry_options_value");
    gtk_editable_delete_text(GTK_EDITABLE(value_entry),0,-1);
  }
  // problem:  need to change entry when GtkListItem's are clicked, not just
  // when the user hits 'enter.'  hold off on this for now.
}

void
on_list_snort_rules_selection_changed  (GtkList         *list,
                                        gpointer         user_data)
{
  currentListItem = NULL;
}



void
on_list_snort_rules_unselect_child     (GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
  clear_interface(NULL);
}


void
on_button_new_rule_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *listbox;

  listbox = lookup_widget(GTK_WIDGET(refholder),"list_snort_rules");
  if (currentListItem != NULL)
    gtk_list_unselect_child(GTK_LIST(listbox), currentListItem);
  else
    clear_interface(NULL);
}


void
on_button_save_ruleset_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
GtkWidget *file_selection;

  if (currentFile != NULL) {
    operation = "save";
    file_prep((const char *)currentFile);
  }
  else {
    operation = "save_as";
    file_selection = create_file_selection();
    gtk_file_selection_complete(GTK_FILE_SELECTION(file_selection),"*.rules");
    gtk_widget_show(file_selection);
  }
}

