#include <stdio.h>
#include <gnome.h>
#include "misc.h"
#include "support.h"
#include "callbacks.h"
#include <stdlib.h>

void
file_opener                          (const char *currentFile)
{
  FILE *input; 
  gchar c[1024];
  gchar *rule_line;
  commentSection = "";
  
  input = fopen((const char *)currentFile, "r+");
  
  if (input == (FILE *)0)
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
			       ("Unable to Open File")));
  else { 
    while ((rule_line = fgets(c, 1023, input)) != (char *)0) {
      if ((rule_line[0] != '#') && (rule_line[0] != '\n'))
	rule_parser(g_strstrip(rule_line));
      else if (rule_line[0] == '#')
       	commentSection = g_strjoin("\n",commentSection, g_strstrip(rule_line), NULL);
    }
    fclose(input);
  }
}

void
rule_parser                          (gchar *rule)
{
  gchar **tempOne, **tempTwo, **tempThree, **tempFour;
  gint i = 0;
  
  tempOne = g_strsplit(rule, "(",2);           // tempOne[0] rule header, tempOne[1] options.
  tempTwo = g_strsplit(tempOne[0], " ", 7);    // each string is a header item.
  tempThree = g_strsplit(tempOne[1], ";",50);
  current.rule_options = g_hash_table_new(g_str_hash, g_str_equal);
  
  if (!g_strcasecmp(operation, "open")) 
    rule_list_filler(rule);
  else if (!g_strcasecmp(operation, "rule_select")) {
    current.action = tempTwo[0];
    current.protocol = tempTwo[1];
    current.src_ip = tempTwo[2];
    current.src_port = tempTwo[3];
    current.direction = tempTwo[4];
    current.dest_ip = tempTwo[5];
    current.dest_port = tempTwo[6];
    while (tempThree[i] != NULL) {
      tempFour = g_strsplit(tempThree[i], ":", 2);
      g_hash_table_insert(current.rule_options, g_strstrip(tempFour[0]), tempFour[1]);
      i++;
    }
  }
}

gchar * rule_builder                    (gpointer *blank)
{
  gchar *headerString;

  headerString = g_strjoin(" ", current.action, current.protocol,
			   current.src_ip, current.src_port, current.direction,
			   current.dest_ip, current.dest_port, NULL);
  optionString = "";
  g_hash_table_foreach(current.rule_options,hash_table_iterate, NULL);
  optionString = g_strjoin(NULL, "(", g_strchomp(optionString), ")", NULL);
  return  g_strjoin(" ", headerString, optionString, NULL);
}

void
rule_list_filler                     (gchar *rule)
{
  GtkWidget *item, *listbox;
  
  listbox = lookup_widget(GTK_WIDGET(refholder), "list_snort_rules");

  if (listbox != NULL) {
    item = gtk_list_item_new_with_label(rule);
    printf("rule is %s in rule_list_filler\n", rule);
    gtk_container_add(GTK_CONTAINER(listbox), item);
    gtk_widget_show(item);
  }
}


void
hash_table_iterate                 (gpointer key,
				    gpointer value,
				    gpointer user_data)
{
  gchar *builder = "";

  if (g_strcasecmp((char *)key,")") == 0)
    ;
  else if ((g_strcasecmp((gchar *)key,"nocase")==0) || (g_strcasecmp((gchar *)key,"sameip")==0) 
	   || (g_strcasecmp((gchar *)key,"regex")==0))
    optionString = g_strconcat(optionString,(gchar *)key, "; ",NULL);
  else {
    builder = g_strconcat((gchar *)key, ":", (gchar *)value,"; ", NULL);
    optionString = g_strconcat(optionString, builder, NULL);
  }
}


void
clear_interface                   (gpointer blank)
{
  GtkWidget *helper;

  helper = lookup_widget(GTK_WIDGET(refholder),"menu_action");
  gtk_option_menu_set_history(GTK_OPTION_MENU(helper),0);

  helper = lookup_widget(GTK_WIDGET(refholder),"menu_protocol");
  gtk_option_menu_set_history(GTK_OPTION_MENU(helper),0);

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_source");
  gtk_editable_delete_text(GTK_EDITABLE(helper),0,-1);

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_src_ports");
  gtk_editable_delete_text(GTK_EDITABLE(helper),0,-1);

  helper = lookup_widget(GTK_WIDGET(refholder),"menu_direction");
  gtk_option_menu_set_history(GTK_OPTION_MENU(helper),0);
  
  helper = lookup_widget(GTK_WIDGET(refholder),"entry_dest");
  gtk_editable_delete_text(GTK_EDITABLE(helper),0,-1);

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_dest_ports");
  gtk_editable_delete_text(GTK_EDITABLE(helper),0,-1);

  helper = lookup_widget(GTK_WIDGET(refholder),"entry_options_value");
  gtk_editable_delete_text(GTK_EDITABLE(helper),0,-1);
}

void
file_prep                       (const char *currentFile)
{
  FILE *output;
  GtkWidget *listbox;
  GList *list_items;
  
  output = fopen((const char *)currentFile,"w");
    
  if (output == (FILE *)0)
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
  			       ("Unable to open file for writing.  Permissions?")));
    else {
      listbox = lookup_widget(GTK_WIDGET(refholder),"list_snort_rules");
      list_items = (GTK_LIST(listbox)->children);
      if (commentSection != NULL) {
	fputs(commentSection, output);
	fputs("\n\n",output);
      }
      g_list_foreach(list_items, file_writer, output);
      fclose(output);
    }
}

void
file_writer                        (gpointer list, gpointer file_handle)
{
  gchar *temp_string, *rule_string;

  gtk_label_get(GTK_LABEL(GTK_BIN(list)->child), &temp_string);
  rule_string = g_strconcat(temp_string,"\n", NULL);
  fputs(rule_string, (FILE *)file_handle);
  
}
